#!/usr/bin/env python
# -*- coding: utf-8 -*-

import time

from twisted.web import http, server
from twisted.internet import protocol
from twisted.internet import reactor
from twisted.internet import defer
from twisted.internet import task
from twisted.web.resource import Resource
from twisted.python import log
from logging import DEBUG
from airi.camera.protocol import CameraFactory
from airi.camera import Listener
from airi import report, RESULT
from functools import partial

MULTIPARTRESPONSE= "--%s\r\nContent-Type: %s\r\nContent-Size: %s\r\n\r\n%s\r\n\r\n"

CATEGORY = "AIRi-Stream"

class MultiPartStream():
  BOUNDARY = 'myBOUNDARY'
  clients = []
  request = None
  oneshot = False

  @report(category=CATEGORY)
  def __init__(self, request):
    self.request = request
    self.request.connectionLost = self.connectionLost
    self.request.multipart=self
    self.oneshot = self.request.args.get('oneshot', ['false',])[0].lower() == 'true'
    MultiPartStream.clients.append(self)

  @classmethod
  def getBoundary(klass):
    return "--%s\r\n" % (klass.BOUNDARY)

  def writeBoundary(self):
    self.request.write(MultiPartStream.getBoundary())

  def writeStop(self):
    self.request.write("%s--\r\n" % MultiPartStream.getBoundary())

  @report(category=CATEGORY)
  def process(self):
    if not self.oneshot:
      self.request.setHeader('Connection', 'Keep-Alive')
      if "flash" in self.request.args:
        self.request.setHeader('Content-Type', 
          'multipart/x-mixed-replace')
      else:
        self.request.setHeader('Content-Type', 
          'multipart/x-mixed-replace; boundary=%s' % (self.BOUNDARY))
    self.request.setHeader('Cache-Control', 'no-cache, no-store, must-revalidate;')
    self.request.setHeader('Expires', '0');
    self.request.setHeader('Pragma-directive', 'no-cache')
    self.request.setHeader('Pragma', 'no-cache')
    self.request.setHeader('Cache-directive', 'no-cache')

  @report(category=CATEGORY)
  def connectionLost(self, reason):
    MultiPartStream.clients.remove(self)

  @report(category=CATEGORY)
  def finish(self):
    try:
      self.request.finish()
    except Exception, err:
      log.err(err)

  def sendPart(self, content, mime="text/html", MULTIPART=None):
    if self.oneshot:
      try:
        self.request.setHeader("Content-Type", mime)
        self.request.setHeader("Content-Size", len(content))
        self.request.write(content)
        self.request.write("\n\n")
        self.finish()
      except Exception, err:
        log.msg("error during sendPart")
        log.err(err)
    else:
      if MULTIPART is None:
        MULTIPART=MULTIPARTRESPONSE % (MultiPartStream.BOUNDARY, mime, len(content), content)
      try:
        self.request.write(MULTIPART)
      except Exception, err:
        log.msg("error during sendPart")
        log.err(err)

  @report(category=CATEGORY)
  @classmethod
  def sendToClients(klass, content, mime="text/html"):
    if len(klass.clients) == 0:
      return

    size = len(content)
    out=MULTIPARTRESPONSE % (klass.BOUNDARY, mime, size, content)

    for client in klass.clients:
      client.sendPart(content, mime, out)

    log.msg("sendToClients, size: %s, count: %s" %(len(content), len(klass.clients)), 
      loglevel=DEBUG,
      category="MultiPartStream")

class StreamResource(Resource, Listener):
  isLeaf = True

  def gotFrame(self, frame, address):
    #print "StreamResource.gotFrame %s" % len(frame)
    size = len(frame)
    out=MULTIPARTRESPONSE % (MultiPartStream.BOUNDARY, "image/jpeg", size, frame)

    for client in MultiPartStream.clients:
      if client.target == address:
        client.sendPart(frame, "image/jpeg")

  @report(category=CATEGORY)
  def lostConnection(self, reason, failed, address):
    print "StreamResource.lostConnection", address
    clients = [ a for a in MultiPartStream.clients if a.target == address ]
    for c in clients:
      c.sendPart(str(reason))
      if not c.oneshot:
        try:
          c.request.finish()
        except Exception, err:
          log.err(err)
      MultiPartStream.clients.remove(c)

  @report(category=CATEGORY, level=RESULT)
  def render_GET(self, request):
    address = request.path.split("/",2)[-1].replace("_", ":")
    request.transport.socket.settimeout(5)
    multipart = MultiPartStream(request)
    multipart.process()
    multipart.target = address
    if len(address) == 17:
      ready = CameraFactory.isConnected(address) or CameraFactory.isPending(address)
      if not ready:
        method = request.args.get("method", ["RFCOMM",])[-1]
        try:
          CameraFactory.connect(address, 1, method)
        except Exception, err:
          log.msg("Failed while trying to connect")
          log.err(err)
    CameraFactory.registerListener(address, self)
    return server.NOT_DONE_YET

class TestStreamResource(Resource):
  isLeaf = True

  def render_GET(self, request):
    log.msg(str(request))
    log.msg(str(request.requestHeaders))
    multipart = MultiPartStream(request)
    multipart.process()
    return server.NOT_DONE_YET

  def render_POST(self, request):
    return self.render_GET(request)

if __name__ == '__main__':
  import webcam
  from twisted.web.server import Site
  import sys
  log.startLogging(sys.stdout)

  webcam.init(False)
  def sendFrame():
    MultiPartStream.sendToClients(webcam.repeat().tostring(), "image/jpeg")
    reactor.callLater(1/15., sendFrame)

  root = Resource()
  root.isLeaf = False
  root.putChild("stream", TestStreamResource())
  reactor.listenTCP(8800, Site(root), interface="0.0.0.0")
  reactor.callLater(0, sendFrame)
  reactor.run()#!/usr/bin/env python



# -*- coding: utf-8 -*-
from twisted.web.resource import Resource
from twisted.python import log
from twisted.web.static import File
from jinja2 import Environment, PackageLoader
from airi.camera.protocol import CameraFactory
from airi.settings import getSettings
import os
import bluetooth

settings = getSettings()

class TemplateResource():
  def __init__(self, template, context):
    self.template = template
    self.context = context

  def render(self, *args, **kwargs):
    return str(self.template.render(self.context))

class Main(Resource):
  isLeaf = False

  def __init__(self, home="/"):
    Resource.__init__(self)
    self.env = Environment(loader=PackageLoader("airi", "templates"))
    self.env.globals["home"]=home

  def index(request):
    return {
      "devices": CameraFactory.getCameras()
    }

  def setup(request):
    if request.method == "POST" and "save" in request.args:
      args = request.args.copy()
      if args["enable_pincode"][0].lower()=="true":
        settings.setPIN(block=args["address"][0], npin=args["pincode"][0])
      else:
        settings.delPIN(args["address"][0])
      for k in ["save", "name", "type", "last", "battery", "status", "enable_pincode", "pincode"]:
        try:
          args.pop(k)
        except:
          pass
      for k in args.keys():
        args[k]=args[k][0]
      print args
      settings.setCamera(args)
      settings.save()

    out = {}
    if "address" not in request.args:
      raise Exception("Invalid address")
    out.update(CameraFactory.getCamera(request.args["address"][-1]))
    print out
    return out

  def scan(request):
    try:
      log.msg("Doing scan")
      cache = bluetooth.discover_devices(lookup_names=True)
      log.msg(cache)
      out = {"devices": 
                [{
                  'address':x[0],
                  'name':x[1],
                  'state': CameraFactory.isConnected(x[0])
                }
                    for x in cache
                ]
            }
      log.msg(out)
      return out
    except Exception, err:
      log.err(err)
      return {"error": str(err)}

  def server_setup(request):
    if request.method == "POST":
      if "delete" in request.args:
        settings.delPIN(request.args["delete"][0])
        settings.save()
      elif "save" in request.args:
        request.args.pop("save")
        blocks = dict([ (b,request.args[b][0]) for b in request.args if b.startswith("block") ])
        pins = dict([ (b,request.args[b][0]) for b in request.args if b.startswith("value") ])
        sets = {}
        for b in blocks:
          if len(blocks[b].strip()) == 0:
            continue
          settings.setPIN(block=blocks[b], npin=pins["value"+b.replace("block","")])
        settings.save()

    out = {}
    out["pins"]=settings.getPINs()
    return out

  def stream(request):
    if "address" not in request.args:
      raise Exception("You need to provide with an address")
    return {
      "isChrome": "chrome" in request.requestHeaders.getRawHeaders("user-agent")[0].lower(),
      "address": request.args["address"][0],
      "camera": CameraFactory.getCamera(request.args["address"][0], True)
    }

  contexts = {
    "index.html": index,
    "setup.html": setup,
    "scan.html": scan,
    "server-setup.html": server_setup,
    "stream.html": stream
  }

  def getChild(self, path, request):
    if "media" in path or "favicon.ico" in path:
      return Resource.getChild(self, path, request)

    if len(path) == 0:
      path = "index.html"
    template = self.env.get_template(path)
    return TemplateResource(template, self.contexts.get(path, lambda x: {})(request))


def main():
  from twisted.application.service import Application
  from twisted.application.internet import TCPServer
  from twisted.web.server import Site
  from twisted.internet import reactor
  from airi.stream import StreamResource
  from airi.api import API
  import sys
  log.startLogging(sys.stdout)

  root = Main()
  path = os.path.dirname(os.path.realpath(__file__))
  root.putChild("api",        API())
  root.putChild("media", File(os.path.join(path, "media/")))
  root.putChild("stream", StreamResource())
  reactor.listenTCP(8000, Site(root), interface="0.0.0.0", backlog=5)
  reactor.run()#!/usr/bin/env python


if __name__=='__main__':
  main()



# -*- coding: utf-8 -*-
from twisted.python import log
from twisted.internet import reactor
from airi.camera import dbg, CameraProtocol
from airi.settings import getSettings

settings=getSettings()

SIZES={
  #day modes
  "QVGA":	1,
  "VGA": 2,
  "XGA": 3,
  "720P": 6,
  "QXGA": 5,

  "QVGA Zoom 1": 11,
  "QVGA Zoom 2": 12,
  "QVGA Zoom 3": 13,
}

PANS = [ "none", "right", "left", "up", "down" ]

CAPABILITIES={ 
  'size':       SIZES.keys(),
  'flash':      True,
  'pan':        PANS,
  'voice':      False, # by now ;)
  'exposure':   True,
  'transport':  ['RFCOMM', 'L2CAP'],
  'battery':    True,
}

class AIRi(CameraProtocol):
  client = None
  callLater = None
  size = "QVGA"
  defaults = {
    "size": "QVGA",
    "flash": 0,
    "voice": False,
    "exposure": 0,
    "pan": "none",
  }

  def getType(self):
    return 'AIRi'

  def __init__(self, client):
    self.client = client
    self.address = client.address
    self.transport = client.transport
    self.doSetup()
  
  def __doCommand(self, command, value):
    dbg("doCommand $%s%s" % (command, value))
    self.transport.write("$%s%s\n\r" % (command, value))
    self.transport.flush()

  def doSetup(self):
    dbg("doSetup")
    nsets = settings.getCamera(self.address)
    sets = self.defaults
    size = self.size
    if nsets:
      sets.update(nsets)
    dbg(sets)
    self.__doCommand("S", SIZES[sets["size"]])
    self.__doCommand("F", "0" if not sets["flash"] else "1")
    if sets["pan"]!="none":
      self.__doCommand("P", sets["pan"])
    self.__doCommand("E", sets["exposure"])

  def updateSettings(self):
    self.doSetup()

  def previewData(self):
    start = self.client.buffer.find("\xff\xd8")
    end = self.client.buffer.find("\xff\xd9")
    if start == -1 or end == -1:
      return

    frame = self.client.buffer[start:end+1]
    self.client.buffer = self.client.buffer[end+2:]
    self.client.gotFrame(frame)

  def dataReceived(self):
    return self.previewData()

  def disconnect(self):
    dbg("AIRi.disconnect")

  def set(self, option, value):
    dbg("set %s-> %s" % (option, value))
    if option == "size":
      self.__doCommand("S", SIZES[value])
    elif option == "flash":
      self.__doCommand("F", "0" if not value else "1")
    elif option == "pan":
      self.__doCommand("P", value)
    elif option == "exposure":
      self.__doCommand("E", value)

AIRi.Capabilities = CAPABILITIES
AIRi.Sizes = SIZES
AIRi.Panning = PANS



#!/usr/bin/python
# -*- coding: utf-8 -*-
#testing pairing service

import gobject

import sys, os
import dbus, dbus.service, dbus.mainloop.glib
from twisted.python import log

PATH="/net/aircable/pairing"

from settings import getSettings
settings=getSettings()

def getPIN(address, dongle):
  return settings.getPIN(address)

def handle_name_owner_changed(own, old, new):
  if own.startswith('org.bluez'):
    if new is None or len(str(new))==0:
      log.msg( "bluez has gone down, time to get out")
    else:
      log.msg( "bluez started, time to restart")

def registerAgent(path):
  adapter = dbus.Interface(bus.get_object("org.bluez", path),
    "org.bluez.Adapter")
  adapter.RegisterAgent(PATH, "KeyboardOnly")
  log.msg("adapter registered for path %s" % path)


def handle_adapter_added(path, signal):
  log.msg("bluez.%s: %s" % (signal, path))
  registerAgent(path)

def handle_adapter_removed(path, signal):
  log.msg("adapter removed %s" % path)

class Rejected(dbus.DBusException):
  _dbus_error_name = "org.bluez.Error.Rejected"

class Agent(dbus.service.Object):
  exit_on_release = True

  def set_exit_on_release(self, exit_on_release):
    self.exit_on_release = exit_on_release

  @dbus.service.method("org.bluez.Agent",
    in_signature="", out_signature="")
  def Release(self):
    log.msg("Agent Release")
    if self.exit_on_release:
      log.msg("Exiting from loop")
      mainloop.quit()

  @dbus.service.method("org.bluez.Agent",
				  in_signature="os", out_signature="")
  def Authorize(self, device, uuid):
      log.msg("Authorize (%s, %s)" % (device, uuid))

  @dbus.service.method("org.bluez.Agent",
    in_signature="o", out_signature="s")
  def RequestPinCode(self, path):
    device = dbus.Interface(bus.get_object("org.bluez", path),
      "org.bluez.Device")
    dongle = dbus.Interface(bus.get_object("org.bluez",
		  device.GetProperties()['Adapter']),
	"org.bluez.Adapter")
    device=str(device.GetProperties()['Address'])
    dongle=str(dongle.GetProperties()['Address'])
    pin=getPIN(device, dongle)
    log.msg("RequestPinCode (%s->%s): %s" % (dongle, device, pin) )
    return pin

  @dbus.service.method("org.bluez.Agent",
    in_signature="o", out_signature="u")
  def RequestPasskey(self, device):
    log.msg("RequestPasskey (%s): %s" % (device, PIN) )
    return dbus.UInt32(PIN)

  @dbus.service.method("org.bluez.Agent",
    in_signature="ou", out_signature="")
  def DisplayPasskey(self, device, passkey):
    log.msg("DisplayPasskey (%s, %d)" % (device, passkey))

  @dbus.service.method("org.bluez.Agent",
    in_signature="ou", out_signature="")
  def RequestConfirmation(self, device, passkey):
    log.msg("RequestConfirmation (%s, %d)" % (device, passkey))
    device = dbus.Interface(bus.get_object("org.bluez", device),
      "org.bluez.Device")
    dongle = dbus.Interface(bus.get_object("org.bluez",
	device.GetProperties()['Adapter']),
      "org.bluez.Adapter")
    device=str(device.GetProperties()['Address'])
    dongle=str(dongle.GetProperties()['Address'])
    pin=getPIN(device, dongle)
    if passkey == pin:
	log.msg("passkey matches")
	return
    log.msg("passkey doesn't match")
    raise Rejected("Passkey doesn't match")

  @dbus.service.method("org.bluez.Agent",
    in_signature="s", out_signature="")
  def ConfirmModeChange(self, mode):
    log.msg("ConfirmModeChange (%s)" % (mode))

  @dbus.service.method("org.bluez.Agent",
    in_signature="", out_signature="")
  def Cancel(self):
    log.msg("Cancel")

def registerControlSignals(bus):
  bus.add_signal_receiver(handle_name_owner_changed,
    'NameOwnerChanged',
    'org.freedesktop.DBus',
    'org.freedesktop.DBus',
    '/org/freedesktop/DBus')

  bus.add_signal_receiver(handle_adapter_added,
    signal_name='AdapterAdded',
    dbus_interface='org.bluez.Manager',
    member_keyword='signal')

  bus.add_signal_receiver(handle_adapter_removed,
    signal_name='AdapterRemoved',
    dbus_interface='org.bluez.Manager',
    member_keyword='signal')

def initAgent():
  try:
    manager = dbus.Interface(bus.get_object("org.bluez", "/"),
						      "org.bluez.Manager")
    for path in manager.ListAdapters():
      registerAgent(path)
    log.msg("Agent registered on all paths")
  except Exception, err:
    log.err("Something went wrong on the agent application")
    log.err(err)

if __name__ == '__main__':
  log.startLogging(sys.stdout)

  dbus.mainloop.glib.DBusGMainLoop(set_as_default=True)

  bus = dbus.SystemBus()
  registerControlSignals(bus)
  mainloop = gobject.MainLoop()
  agent = Agent(bus, PATH)
  agent.set_exit_on_release(False)
  initAgent()
  mainloop.run()
  log.msg("Agent is exiting")



# -*- coding: utf-8 -*-
from twisted.internet.protocol import Protocol, ClientFactory
import airi.twisted_bluetooth as twisted_bluetooth
from twisted.python import log
from airi.camera import dbg, Listener
from twisted.internet import reactor
import optieyes, airicamera
from functools import partial
import bluetooth
from airi.settings import getSettings
from airi import report
from time import localtime, strftime
settings = getSettings()

CATEGORY="airi.camera.protocol"

TYPES = {
  'OPTIEYE': {
      "welcome": "genie cam on",
      "class": optieyes.OptiEye
  },
  'AIRI': {
      "welcome": "$airi",
      "class": airicamera.AIRi
  }
}

def getTagByType(tag):
  out = {}
  for k in TYPES:
    out[k] = TYPES[k][tag]
  return out

WELCOME = getTagByType("welcome")

INIT = 0
IDLE = 1

class Camera(Protocol):
  kind = None
  state = INIT
  callLater = None
  transport = None

  def __init__(self, address):  
    dbg("new camera instance for (%s)", address)
    setattr(self, "client", None)
    setattr(self, "buffer", "")
    self.address = address

  def invalidCamera(self):
    log.msg("not known camera, we stop")
    self.transport.loseConnection()

  def doINIT(self):
    def welcomeCheck():
      while len(self.buffer)>0 and self.buffer.find('\n')>-1:
        welcome, self.buffer = self.buffer.split('\n', 1)
        dbg("welcome", welcome)
        welcome=welcome.strip().lower().strip('\x00')
        for k,v in WELCOME.iteritems():
          if welcome.find(v) > -1:
            return k;

    dbg("doINIT", self.buffer)
    if self.buffer.find("\n") == -1:
      dbg("Welcome not received")
      if not self.callLater:
        self.callLater = reactor.callLater(3, self.invalidCamera)
        #self.addErrback(log.err)
      self.callLater.reset(2)
      return

    kind = welcomeCheck()
    if not kind:
      if not self.callLater:
        self.callLater = reactor.callLater(3, self.invalidCamera)
        #self.callLater.addErrback(log.err)
      self.callLater.reset(2)
      return

    self.kind = kind
    log.msg("Connected to a %s camera" % self.kind)
    dbg("still in buffer", len(self.buffer))
    self.state=IDLE
    settings.setCameraSetting(self.address, "address", self.address)
    settings.setCameraSetting(self.address, "type", self.kind)
    settings.setCameraSetting(self.address, "name", self.getName())
    settings.setCameraSetting(self.address, "last", strftime("%m/%d/%Y %H:%M:%S", localtime()))
    settings.save()
    self.client=TYPES[self.kind]["class"](self)
    if self.callLater:
      self.callLater.cancel()
      self.callLater = None

  def dataReceived(self, data):
    #dbg("dataReceived(%s)" % len(data))
    self.buffer += data
    if self.state == INIT:
      return self.doINIT()
    elif self.client:
      return self.client.dataReceived()

  def getName(self):
    return bluetooth.lookup_name(self.address)

  def __getattr__(self, val):
    dbg("__getattr__ %s" % val)
    if val in [
      'getType', 'getCapabilities', 'getSize', 'getFlash', 'getBattery',
        'getTransport', 'getExposure', 'getZoom', 'getPan', 'getVoice',
        'updateSettings']:
        return getattr(self.client, val, lambda: False)
    raise AttributeError

class CameraFactory(ClientFactory):
  FACTORY = None
  clients = {}
  listeners = {}
  pending = []

  @report(category=CATEGORY)
  def startedConnecting(self, connector):
    pass
#    log.msg("Started to connect.")

  @report(category=CATEGORY)
  def buildProtocol(self, addr):
    log.msg("Connected %s" % str(addr))
    if addr in CameraFactory.pending:
      CameraFactory.pending.remove(addr)
    from airi.api import UpdateManager
    UpdateManager.propagate(addr[0], {"status": True})
    CameraFactory.clients[addr[0]] = Camera(addr[0])
    CameraFactory.clients[addr[0]].gotFrame = partial(CameraFactory.gotFrame, addr[0])
    return CameraFactory.clients[addr[0]]

#@report(category=CATEGORY)

  @classmethod
  def gotFrame(klass, addr, frame):
    #dbg("gotFrame %s" % addr)
    if addr not in klass.listeners:
      #dbg("no one listening for (%s, %s)" % (addr, klass.listeners))
      return
    for listener in klass.listeners[addr]:
      listener.gotFrame(frame, address=addr)

  @classmethod
  @report(category=CATEGORY)
  def setSize(klass, addr, size):
    klass.clients[addr].client.setSize(size)

  @classmethod
  @report(category=CATEGORY)
  def getSize(klass, addr):
    return klass.clients[addr].client.getSize()


  @classmethod
  @report(category=CATEGORY)
  def getSizes(klass, addr):
    return klass.clients[addr].client.getSizes()

  @classmethod
  @report(category=CATEGORY)
  def __cleanup(klass, addr):
    dbg("__cleanup(%s)" % addr)
    if addr in klass.listeners:
      del klass.listeners[addr]
    if addr in klass.clients:
      if getattr(klass.clients[addr], "client", None):
        del klass.clients[addr].client
      del klass.clients[addr]

  @report(category=CATEGORY)
  def __lostConnection(self, addr, reason, failed):
    rep = False

    if addr in CameraFactory.pending:
      CameraFactory.pending.remove(addr)

    if addr in CameraFactory.listeners:
      for listener in CameraFactory.listeners[addr]:
        rep = rep or listener.lostConnection(reason, failed, address=addr)

      if not rep:
        CameraFactory.__cleanup(addr)
    from airi.api import UpdateManager
    UpdateManager.propagate(addr, {"status": False})

    if __name__=='__main__' and reactor.running:
      reactor.stop()

  @report(category=CATEGORY)
  def clientConnectionLost(self, connector, reason):
    addr = connector.getDestination()[0]
    log.msg("Lost connection to %s. Reason: %s" % (addr, reason))
    self.__lostConnection(addr, reason, False)

  @report(category=CATEGORY)
  def clientConnectionFailed(self, connector, reason):
    addr = connector.getDestination()[0]
    log.msg("Connection Failed to %s. Reason: %s" % (addr, reason))
    self.__lostConnection(addr, reason, True)

  @classmethod
  @report(category=CATEGORY)
  def isConnected(klass, addr):
    return addr in klass.clients
    
  @classmethod
  @report(category=CATEGORY)
  def isPending(klass, addr):
    return addr in klass.pending

  @classmethod
  @report(category=CATEGORY)
  def connect(klass, address, channel=1, method="RFCOMM"):
    if klass.isConnected(address):
      raise Exception("All ready connected to %s" % address)
    
    if klass.isPending(address):
      raise Exception("Connection to %s is pending" % address)

    klass.pending.append(address)
    settings = klass.getCamera(address, True)
    method=getattr(settings,"transport", method)
    c = getattr(twisted_bluetooth, "connect%s" % method)

    if klass.FACTORY == None:
      klass.FACTORY=klass()

    if method=="RFCOMM":
      channel=1
    elif method=="L2CAP":
      channel=0x1001
    from airi.api import UpdateManager
    UpdateManager.propagate(address, {"status": "Connecting"})

    c(reactor, address, channel, klass.FACTORY)

  @classmethod
  @report(category=CATEGORY)
  def disconnect(klass, address):
    if klass.isConnected(address):
      try:
        klass.clients[address].client.disconnect()
        klass.clients[address].transport.loseConnection()
      except Exception, err:
        log.msg("call to disconnect failed")
        log.err(err)
    if address in klass.listeners:
      for listener in klass.listeners[address]:
        if getattr(listener, 'forcedDisconnect', None):
          try:
            listener.forcedDisconnect(address=address)
          except:
            pass
    klass.__cleanup(address)

  @classmethod
  @report(category=CATEGORY)
  def registerListener(klass, address, listener):
    if not address in klass.listeners:
      klass.listeners[address] = list()

    if listener not in klass.listeners[address]:
      dbg("adding listener")
      klass.listeners[address].append(listener)

  @classmethod
  @report(category=CATEGORY)
  def removeListener(klass, address, listener):
    try:
      klass.listeners[address].remove(listener)
    except:
      pass

  @classmethod
  @report(category=CATEGORY)
  def getConnected(klass, address=None):
    if address:
      return klass.clients.get(str(address.strip()), None)
    return klass.clients

  @classmethod
  @report(category=CATEGORY)
  def getCamera(klass, address, silent=False):
    out = settings.getCamera(address)
    if not out:
      if not silent:
        raise Exception("Not known camera")
      return out
    out["status"]=klass.isConnected(address)
    out["capabilities"]=TYPES[out["type"]]["class"].Capabilities
    return out

  @classmethod
  @report(category=CATEGORY)
  def getCameras(klass):
    for camera in settings.getCameras():
      camera["status"]=klass.isConnected(camera["address"])
      camera["capabilities"]=TYPES[camera["type"]]["class"].Capabilities
      yield camera

if __name__=='__main__':
  import sys
  class TestListener(Listener):
    def lostConnection(self, reason, failed):
      return False

    def gotFrame(self, frame, *a, **kw):
      print "got frame %s bytes long" % len(frame)
      CameraFactory.disconnect(sys.argv[1])

  if len(sys.argv) != 3:
    print "Usage %s <target> <channel>" % sys.argv[0]
    sys.exit(1)

  log.startLogging(sys.stdout)
  CameraFactory.connect(sys.argv[1], int(sys.argv[2]))
  CameraFactory.registerListener(sys.argv[1], TestListener())
  reactor.run()
